import * as React from "react";
import {observer} from "mobx-react-lite";
import {Grid, makeStyles, Typography} from "@material-ui/core";
import {Conditional, IfFalse, IfTrue, PaddedPaper, WorkstationGenericWrapper} from "@rx-suite/ui";
import {TrackingStationOperatorNoteEntry} from "./TrackingStationOperatorNoteEntry";
import {EerTrackingStationModel} from "../../../../stores/EerTrackingStationStore";
import {MachineModel} from "../../../../stores/MachineStore";
import {Info} from "@material-ui/icons";
import {t, Trans} from "@lingui/macro";
import {grey} from "@material-ui/core/colors";
import {TrackingStationLensEntry} from "./TrackingStationLensEntry";
import {BoxCoaterEventModel} from "../../../../stores/BoxCoaterEventStore";
import {TextBadge} from "../../../generic/TextBadge";

type TrackingStationActionPanelProps = {
    station: EerTrackingStationModel;
    machine: MachineModel;
    eventMaxDuration: number;
}

const useStyles = makeStyles((theme) => ({
    wrapper: {
        width: '80%'
    },
    infoIcon: {
        marginRight: theme.spacing(1),
        color: grey[600]
    },
    infoMessageWrapper: {
        color: grey[600]
    },
    operatorNoteBadge: {
        marginLeft: theme.spacing(1)
    },
}));

export const TrackingStationActionPanel: React.FunctionComponent<TrackingStationActionPanelProps> = observer(({station, machine,  eventMaxDuration}) => {
    const classes = useStyles();

    const shouldDisplayOperatorNoteEntry = React.useCallback((event: BoxCoaterEventModel | null) => {
        if(!event || event.operator_note !== null) {
            return false; // no ongoing event or event already has an operator note
        }
        return event.ongoing_duration > eventMaxDuration;
    }, [station, eventMaxDuration])

    const shouldDisplayLensEntry = React.useCallback((event: BoxCoaterEventModel | null) => {
        if(!event || !event.batch){
            return false; // no ongoing event or batch attached to the event
        }
        return event.batch.lens_count === null || event.batch.empty_holes_count === null;
    }, [])


    if(station.current_event?.is_engineering_lockout_event){
        return null;
    }

    return (
        <WorkstationGenericWrapper WrapperComponent={React.Fragment}>
            <PaddedPaper className={classes.wrapper}>
                <Conditional condition={shouldDisplayOperatorNoteEntry(station.current_event)}>
                    <IfTrue>
                        <TrackingStationOperatorNoteEntry station={station} machine={machine} maxTime={eventMaxDuration}/>
                    </IfTrue>
                    <IfFalse>
                        <Conditional condition={shouldDisplayLensEntry(station.current_event)}>
                            <IfTrue>
                                <TrackingStationLensEntry event={station.current_event as BoxCoaterEventModel}/>
                            </IfTrue>
                            <IfFalse>
                                <Grid container justifyContent={'center'} alignItems={'center'} direction={'column'} style={{height: '100%'}}>
                                    <Grid container item className={classes.infoMessageWrapper} justifyContent={'center'} alignItems={'center'}>
                                        <Info className={classes.infoIcon}/><Typography variant={'h6'}>{t`All information up to date - no action required.`}</Typography>
                                    </Grid>
                                    {station.current_event?.operator_note &&
                                        <Grid item container className={classes.infoMessageWrapper} justifyContent={'center'} alignItems={'center'}>
                                           <Typography variant={'subtitle1'}><Trans>Event logged as </Trans></Typography>
                                            <div className={classes.operatorNoteBadge}>
                                                <TextBadge text={station.current_event.operator_note}/>
                                            </div>
                                        </Grid>
                                    }
                                </Grid>
                            </IfFalse>
                        </Conditional>
                    </IfFalse>
                </Conditional>
            </PaddedPaper>
        </WorkstationGenericWrapper>
    )
})
