import * as React from "react";
import {observer} from "mobx-react-lite";
import {OperatorNoteModel, OperatorNoteStore} from "../../../../stores/OperatorNoteStore";
import {OperatorNoteGroupStore} from "../../../../stores/OperatorNoteGroupStore";
import {PaginatedButtonGridWithSortControls} from "../../../generic/PaginatedButtonGridWithSortControls";
import {useSnackbar} from "notistack";
import {EerTrackingStationModel} from "../../../../stores/EerTrackingStationStore";
import {ModelProvider} from "@rx-suite/ui";
import {MachineModel} from "../../../../stores/MachineStore";
import {TrackingStationReasonDoorMessage} from "./TrackingStationReasonDoorMessage";
import {BoxCoaterEventModel} from "../../../../stores/BoxCoaterEventStore";
import {t} from "@lingui/macro";

type OperatorNoteEntryProps = {
    station: EerTrackingStationModel;
    machine: MachineModel;
    maxTime: number;
}

export const TrackingStationOperatorNoteEntry: React.FunctionComponent<OperatorNoteEntryProps> = observer(({station, machine, maxTime}) => {
    const {enqueueSnackbar} = useSnackbar();

    const reasonButtons = React.useMemo(() => {
        const event = station.current_event

        if(event === null){
            return;
        }

        // Filter by group first
        let operatorNotes = OperatorNoteStore.models.filter(n => station.operator_note_group_ids.includes(n.operator_note_group_id));

        // Filter by event type
        operatorNotes = operatorNotes.filter(n => {
            if(event.is_door_open_event){
                return n.display_during_door_open;
            } else if(event.is_door_closed_event){
                return n.display_during_door_closed;
            } else{
                return false;
            }
        });


        return operatorNotes.map(note => {
            return {
                label: note.code,
                data: note,
                groupId: note.operator_note_group_id
            }
        })
    }, [station.current_event, OperatorNoteStore.models]);

    const groupButtons = React.useMemo(() => {
        const operatorNoteGroups = OperatorNoteGroupStore.models.filter(n => station.operator_note_group_ids.includes(n.id));

        return operatorNoteGroups.map(group => {
            return {
                label: group.name,
                groupId: group.id as number
            }
        })
    }, [OperatorNoteGroupStore.models]);

    const handleNoteEntry = React.useCallback(async (note: OperatorNoteModel) => {
        if(!station.current_event){
            return;
        }

        station.current_event.viewModel.operator_note = note.code;
        station.current_event.viewModel.submit();

        try{
            await station.current_event.submitEventInformation();
            enqueueSnackbar(t`Operation Code successfully logged.`, {variant: "success"})
        }catch (e) {
            console.error(e);
            enqueueSnackbar(t`Error while saving!`, {variant: "error"})
        }
    }, [station.current_event]);

    return (
        <React.Fragment>
            <ModelProvider value={station}>
                <TrackingStationReasonDoorMessage event={station.current_event as BoxCoaterEventModel} elapsedTime={maxTime}/>
                <PaginatedButtonGridWithSortControls
                    buttonsData={reasonButtons}
                    sortButtonsData={groupButtons}
                    submitAction={handleNoteEntry}
                    buttonsPerPage={12}
                    buttonsPerRow={4}
                />
            </ModelProvider>
        </React.Fragment>
    )
})
