import React, {useMemo} from "react";
import {observer} from "mobx-react-lite";
import {Button, IconButton, Divider, CircularProgress} from "@material-ui/core";
import Grid from "@material-ui/core/Grid";
import {ChevronLeft, ChevronRight} from "@material-ui/icons";
import {makeStyles} from "@material-ui/core/styles";
import {Conditional, IfFalse, IfTrue} from "@rx-suite/ui";
import {Model} from "@rx-suite/data/build";

interface PaginatedButtonGroupProps {
    buttonsData: ButtonGridButtonData[];
    sortButtonsData?: ButtonGridSortButtonData[];
    buttonsPerPage: number;
    buttonsPerRow: number;
    submitAction: (data: any) => Promise<void>;
}

interface ButtonGridSortButtonData  {
    label: string;
    groupId: number;
}

interface ButtonGridButtonData  {
    label: string;
    data: Model;
    groupId?: number;
}

const useStyles = makeStyles((theme) => ({
    divider: {
        margin: theme.spacing(2),
    },
    page: {
        margin: theme.spacing(3),
    },
    buttonGroup: {
        minHeight: (75+(theme.spacing(2)*2))*3,
    },
    groupButton: {
        height: '50px',
        minWidth: '75%',
        fontSize: '1rem'
    },
    actionButton: {
        minWidth: '75%',
        minHeight: '75px',
        margin: theme.spacing(1),
        fontSize: '1.175rem'
    },
    operatorNoteLabel: {
        overflow: 'hidden',
        textOverflow: 'ellipsis'
    }
}));

export const PaginatedButtonGridWithSortControls: React.FunctionComponent<PaginatedButtonGroupProps> = observer(({buttonsData, sortButtonsData, buttonsPerPage, buttonsPerRow, submitAction}) => {
    const classes = useStyles();

    const [currentPageNum, setCurrentPageNum] = React.useState(0);
    const [currentGroup, setCurrentGroup] = React.useState<undefined | number>(sortButtonsData ? sortButtonsData[0].groupId : undefined);
    const [isProcessing, setIsProcessing] = React.useState(false);

    const sortButtons = React.useMemo(()=> {
        if(!sortButtonsData){
            return []
        }
        return sortButtonsData;
    }, [sortButtonsData])


    const currentActionButtons = React.useMemo(()=> {
        let data = buttonsData.sort((a,b) => a.label.localeCompare(b.label));

        if(currentGroup){
            data = data.filter(b => b.groupId === currentGroup);
        }

        const first = currentPageNum *  buttonsPerPage;
        const last  = first + buttonsPerPage;
        const pageCount = Math.ceil(data.length / buttonsPerPage);

        if(last > data.length){
            data = data.slice(first, data.length);
        }else{
            data = data.slice(first, last);
        }

        return {
            pageCount: pageCount,
            buttons: data
        }
    }, [currentPageNum, buttonsData, currentGroup, buttonsPerPage])

    const handleSubmit = React.useCallback(async (data)=> {
        setIsProcessing(true);
        console.log(data)
        await submitAction(data);
        setIsProcessing(false);
    }, [submitAction])

    const handleGroupChange = React.useCallback(newValue => {
        setCurrentPageNum(0)
        setCurrentGroup(newValue);
    }, []);

    return (
        <div className={classes.page}>
            <Conditional condition={sortButtons.length > 0}>
                <IfTrue>
                    <Grid container justify="center">
                        {
                            sortButtons.map(data => {
                                return <Grid key={data.groupId} container item justify="center" xs={2}>
                                            <Button
                                                className={classes.groupButton}
                                                size="large" variant={data.groupId === currentGroup ? "contained" : "outlined"}
                                                color="primary"
                                                onClick={() => { handleGroupChange(data.groupId) }}
                                                disableElevation
                                                disabled={isProcessing}>
                                                    {data.label}
                                            </Button>
                                        </Grid>
                            })
                        }
                    </Grid>
                </IfTrue>
            </Conditional>
            <Divider className={classes.divider}/>
            <Grid className={classes.buttonGroup} container justify="center" >
                <Grid item container justify="center" xs={1}>
                    <IconButton disabled={currentPageNum === 0}
                                onClick={()=> setCurrentPageNum(currentPageNum - 1)}>
                        <ChevronLeft/>
                    </IconButton>
                </Grid>
                <Grid item container justify="center" xs={10}>
                    <Conditional condition={isProcessing}>
                        <IfTrue>
                            <Grid item container justify="center" alignItems="center">
                                <CircularProgress/>
                            </Grid>
                        </IfTrue>
                        <IfFalse>
                            {
                                currentActionButtons.buttons.map(button => {
                                    return <Grid key={button.data.getId()} item container justify="center" alignItems="center" style={{width: `${(100/buttonsPerRow)}%`}} >
                                                <Button
                                                    onClick={async () => await handleSubmit(button.data) }
                                                    className={classes.actionButton}
                                                    size="large"
                                                    variant="contained"
                                                    color="primary"
                                                    disableElevation
                                                >
                                                    <div className={classes.operatorNoteLabel}>{button.label}</div>
                                                </Button>
                                            </Grid>})
                            }
                        </IfFalse>
                    </Conditional>
                </Grid>
                 <Grid item container justify="center" xs={1} >
                    <IconButton disabled={currentPageNum === (currentActionButtons.pageCount - 1)}
                                onClick={()=> setCurrentPageNum(currentPageNum + 1)}>
                        <ChevronRight/>
                    </IconButton>
                </Grid>
            </Grid>
        </div>
    )
});
