import * as React from "react";
import {ButtonField, FormProps, makeColumn, ModelTable, SelectField} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {OperatorNoteModel, OperatorNoteStore} from "../../stores/OperatorNoteStore";
import {t} from "@lingui/macro"
import {OperatorNoteForm} from "./OperatorNoteForm";
import {useSnackbar} from "notistack";
import {FetchError, Model} from "@rx-suite/data/build";
import {EerCategoryStore} from "../../stores/EerCategoryStore";
import {createStyles, IconButton, makeStyles, Theme} from "@material-ui/core";
import {Filter, ReactTableFunction} from "react-table";
import {OperatorNoteCategoryBadge} from "../eer_category/OperatorNoteCategoryBadge";
import {faPen} from "@rx-suite/pro-solid-svg-icons";
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        button: {
            padding: '12px'
        }
    })
);

export const OperatorNotesBreadModel: React.FunctionComponent<FormProps> = observer(({model}) => {
    const {enqueueSnackbar} = useSnackbar();
    const classes = useStyles();


    const [addOperatorNoteOpen, setAddOperatorNoteOpen] = React.useState(false)
    const [selectedOperatorNoteModel, setSelectedOperatorNoteModel] = React.useState<OperatorNoteModel>(OperatorNoteStore.createModel());

    const handleAddOperatorNoteSubmit = React.useCallback((submitModel: Model) => {
        submitModel.viewModel.operator_note_group_id = model.getId();
        submitModel.viewModel.submit();
        submitModel.commit().then(
            () => {
                enqueueSnackbar(t`Operator note successfully created.`, {variant: "success"})
                setAddOperatorNoteOpen(false);
            },
            (e: FetchError)=> {
                enqueueSnackbar(t`Something went wrong while saving the operator note.`, {variant: "error"})
            }
        );
    }, []);

    const handleAddOperatorNoteClose = React.useCallback(() => {
        setAddOperatorNoteOpen(false);
    }, []);

    const handleClickOpen = React.useCallback((model: OperatorNoteModel) => {
        setSelectedOperatorNoteModel(model);
        setAddOperatorNoteOpen(true);
    }, [model]);

    const columns = React.useMemo(() => {
        return [
            makeColumn('code', t`Operation Code`, 'string'),
            {
                Header: t`Display During`,
                id: 'display_group',
                accessor: d => d,
                filterable: true,
                Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction}) => (
                    <SelectField
                        options={
                            [
                                {label: t`Door Close and Door Open`, value: 'both'},
                                {label: t`Door Close`, value: 'closed'},
                                {label: t`Door Open`, value: 'open'}
                            ]
                        }
                        onChange={(name, value) => onChange(value)}
                        value={filter ? filter.value : ""}
                    />
                ),
                Cell: (cellInfo: {value: OperatorNoteModel }) => {
                    const displayType = cellInfo.value.display_group
                    const display = t`Door Open`;

                    if(displayType === 'both'){
                        return t`Door Close and Door Open`;
                    } else if(displayType === 'closed') {
                        return t`Door Close`
                    }

                    return (
                        <>{display}</>
                    )}
            },
            {
                Header: t`Report Category`,
                id: 'eer_category_id',
                accessor: d => d,
                filterable: true,
                Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction}) => (
                    <SelectField
                        options={
                            EerCategoryStore.models.map((category) => {
                                return {label: category.name, value: category.id}
                            })
                        }
                        onChange={(name, value) => onChange(value)}
                        value={filter ? filter.value : ""}
                    />
                ),
                Cell: (cellInfo: {value: OperatorNoteModel }) => {
                    return (<OperatorNoteCategoryBadge model={cellInfo.value}/>)
                }
            },
            makeColumn('description', t`Description`, 'string'),
        ]
    }, [model.eer_category_id, EerCategoryStore.models])

    return (
        <div>
            <ButtonField text={t`Add Operator Note`} onClick={()=>{handleClickOpen(OperatorNoteStore.createModel())}} variant={"outlined"} color={"secondary"}/>
            <ModelTable store={OperatorNoteStore}
                        columns={columns}
                        showCreateButton={false}
                        defaultFiltered={[{id: 'operator_note_group_id', value: model.getId()}]}
                        defaultSorted={[{id: 'code', desc: false}]}
                        replacementActions={
                            {
                                Edit: ({model}) =>
                                    <IconButton onClick={() => handleClickOpen(model as OperatorNoteModel)} size={"small"} className={classes.button}>
                                        <FontAwesomeIcon icon={faPen}/>
                                    </IconButton>,
                                Read: () => null
                            }
                        }
                        suppressWrapper
                        />
            {open &&
            <OperatorNoteForm open={addOperatorNoteOpen} handleSubmit={handleAddOperatorNoteSubmit} handleClose={handleAddOperatorNoteClose} model={selectedOperatorNoteModel}/>}
        </div>
    )
})