import * as React from "react";
import {ModelInput, ModelForm, ModelSelect, HintButton} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {MachineStore} from "../../../stores/MachineStore";
import {Grid, LinearProgress, makeStyles} from "@material-ui/core";
import {OperatorNoteGroupStore} from "../../../stores/OperatorNoteGroupStore";
import {EerTrackingStationModel, EerTrackingStationStore} from "../../../stores/EerTrackingStationStore";
import {t, Trans} from "@lingui/macro";
import TextField from "@material-ui/core/TextField";

type EerTrackingStationFormProp = {
    model: EerTrackingStationModel;
}

const useStyles = makeStyles((theme) => ({
    timeSelect: {
        flexGrow: 1,
        marginTop: theme.spacing(1),
        marginBottom: theme.spacing(2),
    }
}));


export const EerTrackingStationForm: React.FunctionComponent<EerTrackingStationFormProp> = observer(({model}) => {
    const classes = useStyles();

    const dataMachines = MachineStore.getAll();
    const operatorNoteGroups = OperatorNoteGroupStore.getAll();
    const eerTrackingStations = EerTrackingStationStore.getAll();

    if(MachineStore.isLoading || OperatorNoteGroupStore.isLoading) {
        return <LinearProgress/>
    }

    const dataMachineOptions = dataMachines.map((dataMachine) => {
        const alreadyMachineWithId = eerTrackingStations.some((station) => station.rx_data_machine_id === dataMachine.id)
        return {label: dataMachine.extendedName, value: dataMachine.getId(), isDisabled: alreadyMachineWithId}
    })

    const noteGroupOptions = operatorNoteGroups.map((noteGroup) => {
        return {label: noteGroup.name, value: noteGroup.getId()}
    })

    const assign_machine_id = (name: string, value: string) => {
        model.nestedAssignment(name, value);
        const dataMachine = dataMachines.find((dataMachine) => dataMachine.getId() === value);
        if(dataMachine) {
            model.nestedAssignment('machine_sn', dataMachine.sn);
        }
    }

    return (
        <ModelForm model={model} suppressMakeCopy suppressImportExport>
            <ModelInput name={'name'}/>
            <ModelSelect label={t`Associate to Machine`} name={'rx_data_machine_id'}
                         options={dataMachineOptions}
                         onChange={assign_machine_id}
                         onSubmit={assign_machine_id}
                         disabled={!model.isNew}
                         hint={model.isNew ? null : {title: t`Associate to Machine`, text: t`Once a tracking station is created for a machine, the associated machine cannot be changed.`}}
            />
            <ModelSelect name={'operator_note_group_ids'} options={noteGroupOptions} multiple={true}/>
            <Grid container>
                <TextField
                    className={classes.timeSelect}
                    name={'door_open_max_time'}
                    label={t`Door Open Max Time (minutes)`}
                    value={model.viewModel.door_open_max_time/60}
                    onChange={(e)=>{model.viewModel.door_open_max_time = (e.target.value * 60)}}
                    type={'number'}
                    inputProps={
                        {
                            min: 1,
                        }
                    }
                />
            </Grid>
            <Grid container alignItems={'center'}>
                <TextField
                    className={classes.timeSelect}
                    name={'door_closed_max_time'}
                    label={t`Default Door Closed Max Time (minutes)`}
                    value={model.viewModel.door_closed_max_time/60}
                    onChange={(e)=>{model.viewModel.door_closed_max_time = (e.target.value * 60)}}
                    type={'number'}
                    inputProps={
                        {
                            min: 1,
                        }
                    }
                />
                <HintButton title={t`Default Door Closed Max Time`}>
                    <Trans>The default door closed max time is only used if the recipe currently processed by the box coater is unknown.</Trans>
                </HintButton>
            </Grid>
        </ModelForm>
    )
})
