import * as React from "react";
import {observer} from "mobx-react-lite";
import {WorkstationHelper} from "@rx-suite/ui";
import {EerTrackingStationModel} from "../../../stores/EerTrackingStationStore";
import {TrackingStationMachineInfoPanel} from "../../../components/err_tracking_workstation/panels/machine_info_panel/TrackingStationMachineInfoPanel";
import {Box, Grid, LinearProgress, Typography} from "@material-ui/core";
import {TrackingStationEventTimerPanel} from "../../../components/err_tracking_workstation/panels/event_timer_panel/TrackingStationEventTimerPanel";
import {TrackingStationBatchInfoPanel} from "../../../components/err_tracking_workstation/panels/batch_info_panel/TrackingStationBatchInfoPanel";
import {TrackingStationActionPanel} from "../../../components/err_tracking_workstation/panels/action_panel/TrackingStationActionPanel";
import {MachineStore} from "../../../stores/MachineStore";
import {OperatorNoteGroupStore} from "../../../stores/OperatorNoteGroupStore";
import {OperatorNoteStore} from "../../../stores/OperatorNoteStore";
import {BoxCoaterEventModel} from "../../../stores/BoxCoaterEventStore";
import {BoxCoaterRecipeStore} from "../../../stores/BoxCoaterRecipeStore";
import {Trans} from "@lingui/macro";
import {Alert} from "@material-ui/lab";

type EerTrackingWorkstationProps = {
    model: EerTrackingStationModel;
}

export const EerTrackingWorkstation: React.FunctionComponent<EerTrackingWorkstationProps> = observer(({model}) => {
    React.useEffect(() => {
        model.fetchFullModel();
        OperatorNoteGroupStore.getAll();
        OperatorNoteStore.getAll();
        BoxCoaterRecipeStore.getAll();
    }, [])

    const machine = React.useMemo(()=> {
        return MachineStore.models.find(m => m.id == model.rx_data_machine_id);
    }, [MachineStore.models])

    if(!machine || !model.loaded_full_model){
        return <LinearProgress/>
    }

    const invalidRecipe = React.useMemo(()=> {
        const event = model.current_event;
        if (!event || event.is_engineering_lockout_event) {
            return null;
        }

        const recipeName = event.recipe;
        if (!recipeName) {
            return null;
        }

        const recipe = BoxCoaterRecipeStore.models.find(r => r.recipe_code === recipeName);
        if (recipe) {
            return null;
        }

        return (
            <Alert severity={'warning'}>
                <Trans>{`Processing time of recipe code "${recipeName}" is unknown, using default time.`}</Trans>
            </Alert>
        );

    }, [model.current_event, BoxCoaterRecipeStore.models])

    const eventMaxDuration = React.useCallback((event: BoxCoaterEventModel) => {
        if (!event || event.is_engineering_lockout_event) {
            return 0;
        }

        if (event.is_door_open_event) {
            return model.door_open_max_time;
        }

        const recipeName = event.recipe;

        if (recipeName) {
            const recipe = BoxCoaterRecipeStore.models.find(r => r.recipe_code === recipeName);
            if (recipe) {
                return recipe.processing_time;
            }
        }

        return model.door_closed_max_time;

    }, [model, BoxCoaterRecipeStore.models])


    return (
        <WorkstationHelper stores={[]}>
                <Box mb={1}>
                    {invalidRecipe}
                    <Typography variant={'h3'}>{machine.extendedName}</Typography>
                </Box>
                <Grid container>
                    <TrackingStationMachineInfoPanel station={model} machine={machine}/>
                    {model.current_event && <TrackingStationEventTimerPanel event={model.current_event} eventMaxDuration={eventMaxDuration(model.current_event)}/>}
                </Grid>
                <Box mt={1}>
                    <Grid container>
                        {model.current_event && <TrackingStationBatchInfoPanel event={model.current_event}/>}
                        {model.current_event && <TrackingStationActionPanel machine={machine} station={model} eventMaxDuration={eventMaxDuration(model.current_event)}/>}
                    </Grid>
                </Box>
        </WorkstationHelper>
    )
})