import * as React from "react";
import {BreadModel, makeColumn, SelectField} from "@rx-suite/ui";
import {t} from "@lingui/macro"
import {BoxCoaterEventModel, BoxCoaterEventStore} from "../../stores/BoxCoaterEventStore";
import {BoxCoaterEventForm} from "./BoxCoaterEventForm";
import {Filter, ReactTableFunction} from "react-table";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {MachineNameBadge} from "../../components/generic/MachineNameBadge";
import {MachineStore} from "../../stores/MachineStore";
import {BreadModelBatchId} from "../../components/box_coater_events/bread_model/BreadModelBatchId";
import {BreadModelActionBadges} from "../../components/box_coater_events/bread_model/BreadModelActionBadges";
import {BoxCoaterRecipeStore} from "../../stores/BoxCoaterRecipeStore";

export const BoxCoaterEventsBreadModel = () => {
    BoxCoaterRecipeStore.getAll();
    const columns = React.useMemo(() => {
        return [
            {
                Header: t`Time`,
                id: 'time',
                accessor: (model: BoxCoaterEventModel) => model,
                filterable: true,
                Filter: ({filter, onChange}: { filter: Filter, onChange: ReactTableFunction }) => (
                    <SelectField
                        options={
                            [
                                {label: t`Today`, value: 'day'},
                                {label: t`This Week`, value: 'week'},
                                {label: t`This Month`, value: 'month'}
                            ]
                        }
                        onChange={(name, value) => onChange(value)}
                        value={filter ? filter.value : 'day'}
                    />
                ),
                Cell: (cellInfo: { value: BoxCoaterEventModel }) => {
                    const settings = SystemSettingStore.selectedModel as SystemSettingModel;
                    return (
                        <>
                            {settings ? settings.printDateTime(cellInfo.value.time) : cellInfo.value.time}
                        </>
                    )
                }
            },
            {
                Header: t`Machine`,
                id: 'machine_id',
                filterable: true,
                accessor: (model: BoxCoaterEventModel) => model,
                Filter: ({filter, onChange}: { filter: Filter, onChange: ReactTableFunction }) => (
                    <SelectField
                        options={
                            MachineStore.models.map((machine) => {
                                return {label: machine.extendedName, value: machine.id}
                            })
                        }
                        onChange={(name, value) => onChange(value)}
                        value={filter ? filter.value : ""}
                    />
                ),
                Cell: (cellInfo: {value: BoxCoaterEventModel }) => {
                    return (<MachineNameBadge machineId={cellInfo.value.machine_id}/>)
                }
            },
            {
                Header: t`Event`,
                id: 'event_type',
                filterable: true,
                accessor: (model: BoxCoaterEventModel) => model,
                Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction}) => (
                    <SelectField
                        options={
                            [
                                {label: t`All`, value: ""},
                                {label: t`Door Opened`, value: 0},
                                {label: t`Door Closed`, value: 1},
                                {label: t`Engineering Lockout`, value: 2}
                            ]
                        }
                        onChange={(name, value) => onChange(value)}
                        value={filter ? filter.value : ""}
                    />
                ),
                Cell: (cellInfo: {value: BoxCoaterEventModel }) => {
                    return (
                        <>
                            {cellInfo.value.resolved_event_type}
                        </>
                    )}
            },
            {
                Header: t`Duration`,
                id: 'duration',
                accessor: (model: BoxCoaterEventModel) => model,
                Cell: (cellInfo: {value: BoxCoaterEventModel }) => {
                    return (
                        <>
                            {cellInfo.value.resolved_duration}
                        </>
                    )}
            },
            {
                Header: t`Batch Id`,
                id: 'batch_id',
                filterable: true,
                sortable: false,
                accessor: (model: BoxCoaterEventModel) => model,
                Cell: (cellInfo: {value: BoxCoaterEventModel }) => {
                    return (
                        <BreadModelBatchId event={cellInfo.value}/>
                    )}
            },
            {accessor: d => d, Header: t`Batch Recipe`, id: 'recipe_name', Cell: (cellInfo: { value: BoxCoaterEventModel }) => {

                    const model = cellInfo.value;
                    if (!model || !model.getId) {
                        return null;
                    }

                    const recipeName = model.recipe;
                    if (!recipeName) {
                        return null;
                    }

                    const recipe = BoxCoaterRecipeStore.models.find(r => r.recipe_code === recipeName);
                    if (!recipe) {
                        return t`unknown (${recipeName})`;
                    }

                    return recipe.name;
                }},
            {
                Header: t`Action Required?`,
                id: 'action_required',
                filterable: false,
                sortable: false,
                accessor: (model: BoxCoaterEventModel) => model,
                Cell: (cellInfo: {value: BoxCoaterEventModel }) => {
                    return (
                        <BreadModelActionBadges event={cellInfo.value}/>
                    )}
            }
        ]
    }, [SystemSettingStore.selectedModel, MachineStore.models])

    return (
        <BreadModel
            store={BoxCoaterEventStore}
            Form={BoxCoaterEventForm}
            columns={columns}
            tableProps={
                {
                    replacementActions: {
                        Delete: () => null
                    },
                    defaultSorted: [
                        {id: 'time', desc: true}
                    ],
                    defaultFiltered: [
                        {id: 'time', value: 'day'}
                    ]
                }
            }
        />
    )
}