import {attribute, DataStore, Model, model, Types} from "@rx-suite/data";
import * as yup from "yup";
import {t} from "@lingui/macro"
import {action, computed} from "mobx";
import * as React from "react";
import {BoxCoaterBatchModel} from "./BoxCoaterBatchStore";
import {TimeModel, TimeStore} from "@rx-suite/stores";
import moment from "moment";

@model({
    transportLayer: "/api/box_coater_events",
    routingInfo: {
        list: '/box_coater_events',
        read: '/box_coater_events/:id/edit',
        update: '/box_coater_events/:id/edit'
    }
})

export class BoxCoaterEventModel extends Model {
    static types: Types = {
        batch: 'object'
    };

    static relationships = {
        batch: BoxCoaterBatchModel
    };

    @attribute({label: t`Id`, validation: yup.number()})
    id: number = 0;

    @attribute({label: t`Time`})
    time: string = "";

    @attribute()
    machine_id: number = -1;

    @attribute({label: t`Event Type`})
    event_type: 'door_open' | 'door_closed' | 'engineering_lockout' = 'door_open';

    @attribute({label: t`Recipe Name`})
    recipe: string | null = null;

    @attribute({label: t`Operator Note`})
    operator_note: string | null = '';

    @attribute({label: t`Reporting Category`})
    reporting_category: string = '';

    @attribute({label: t`Duration`})
    duration: number | null = null;

    @attribute()
    batch: BoxCoaterBatchModel | null = null;

    // Note: the current event will always show as NOT needing an operator note as that must be calculated using the elapsed time
    @attribute({label: t`Requires Performance Note `})
    requires_operator_note: boolean = false;

    @computed
    get ongoing_duration() {
        if(this.duration){
            return this.duration;
        }

        const serverTime = TimeStore.selectedModel as TimeModel;
        if(!serverTime){
            return 0;
        }

        // Return the difference in seconds
        return Math.ceil(moment.duration(serverTime.currentServerTime.diff(moment(this.time))).asSeconds());
    }

    @computed
    get resolved_duration() {
        if (!this.duration) {
            return t`ongoing`;
        }
        if (this.duration < 60) {
            return t`less than a minute`;
        }
        if (this.duration < 3600){
            const minutes = Math.floor(this.duration/60);
            return minutes == 1 ? t`1 minute` : t`${minutes} minutes`;
        }
        if (this.duration < 36000){
            const hours = Math.floor(this.duration/3600)
            const minutes = Math.floor((this.duration % 3600)/60)

            const hourText = hours == 1 ? t`1 hour` : t`${hours} hours`;
            let minText = "";
            if(minutes > 0){
                minText = ", " + (minutes == 1 ? t`1 minute` : t`${minutes} minutes`);
            }

            return hourText + minText
        }
        return t`more than a day`;
    }

    @computed
    get resolved_event_type() {
        if (this.event_type == 'door_closed') {
            return t`Door Closed`;
        }
        if (this.event_type == 'door_open') {
            return t`Door Opened`;
        }
        if (this.event_type == 'engineering_lockout') {
            return t`Engineering Lockout`;
        }
        return t`Unknown`;
    }

    @computed
    get is_door_open_event() {
        return this.event_type === 'door_open'
    }

    @computed
    get is_door_closed_event() {
        return this.event_type === 'door_closed'
    }

    @computed
    get is_engineering_lockout_event() {
        return this.event_type === 'engineering_lockout'
    }

    @action
    async submitEventInformation() {
        const resp = await fetch(`${this.store.transportLayer.base}/${this.id}`, {
            method: "PUT",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify(
                {
                        box_coater_event: {
                            id: this.id,
                            operator_note: this.operator_note,
                            reporting_category: this.reporting_category,
                            recipe: this.recipe,
                            batch: this.batch ?
                            {
                                id: this.batch.id,
                                lens_count: this.batch.lens_count,
                                empty_holes_count: this.batch.empty_holes_count
                            } : null
                        }
                    }
            )
        })

        const data = await resp.json();
        if (!resp.ok) {
            throw data.error;
        }
    }

    constructor() {
        super();
        super.setup();
    }
}

export const BoxCoaterEventStore = BoxCoaterEventModel.store as DataStore<BoxCoaterEventModel>;