import {attribute, DataStore, Model, model, Types} from "@rx-suite/data";
import * as yup from "yup";
import {t} from "@lingui/macro"
import {action, computed} from "mobx";
import {BoxCoaterEventModel} from "./BoxCoaterEventStore";
import {OperatorNoteModel} from "./OperatorNoteStore";

@model({
    transportLayer: "/api/eer_tracking_stations",
    permissionTarget: "EerTrackingStation",
    routingInfo: {
        list: '/settings/eer_tracking_stations',
        create: '/settings/eer_tracking_stations/create',
        read: '/settings/eer_tracking_stations/:id/edit',
        update: '/settings/eer_tracking_stations/:id/edit'
    }
})

export class EerTrackingStationModel extends Model {

    static types: Types = {
        current_event: 'object'
    };

    static relationships = {
        current_event: BoxCoaterEventModel
    };

    @attribute({label: t`Id`, validation: yup.number()})
    id: number = 0;

    @attribute({label: t`Name`, validation: yup.string().min(3, t`Enter name to identify this box coater in the side menu.`)})
    name: string = '';

    @attribute({label: t`Door Open Max Time`, validation: yup.number().positive().integer().min(60)})
    door_open_max_time: number = 300;

    @attribute({label: t`Default Door Closed Max Time`, validation: yup.number().positive().integer().min(60)})
    door_closed_max_time: number = 1500;

    @attribute({label: t`Associated Machine`, validation: yup.number().min(1, t`This tracking station must be associated to a machine.`)})
    rx_data_machine_id: number = 0;

    @attribute({label: t`Operator Note Groups`, validation: yup.array().min(1, t`At least 1 operator note group must be selected.`)})
    operator_note_group_ids: any[] = [];

    @attribute({label: t`Locked`, validation: yup.boolean()})
    locked: boolean = false;

    @attribute()
    current_event: BoxCoaterEventModel | null = null;

    @attribute()
    loaded_full_model: boolean = false;

    @action
    async fetchFullModel() {
        if(this.loaded_full_model){
            return;
        }

        const resp = await fetch(`${this.store.transportLayer.base}/${this.id}`, {
            method: "GET",
            headers: {'Content-Type': 'application/json'}
        })
        const data = await resp.json();

        if (!resp.ok) {
            throw data.error;
        }
        const model = this.store.createModel(data);
        this.store.setModel(model, true);
    }

    @action
    async toggleEngineeringLockout() {
        if(!this.current_event){
            return;
        }

        const resp = await fetch(`${this.store.transportLayer.base}/${this.id}/engineering_lockout`, {
            method: "PUT",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({lock: !this.current_event.is_engineering_lockout_event})
        })
        const data = await resp.json();

        if (!resp.ok) {
            throw data.error;
        }

        const model = this.store.createModel(data);
        this.store.setModel(model, true);
    }

    constructor() {
        super();
        super.setup();
    }
}

export const EerTrackingStationStore = EerTrackingStationModel.store as DataStore<EerTrackingStationModel>;